(() => {
  const simplerInit = () => {
    Array.from(document.querySelectorAll('a.simpler-checkout')).forEach(el => {
      if (el.classList.contains('listening')) {
        return false;
      }
      const checkout = el.classList.contains('simpler-checkout-single') ? new SimplerCheckoutSingle() : new SimplerCheckout();
      el.classList.add('listening');
      el.addEventListener('click', e => checkout.checkout(e));
    });
  };

  if (document.readyState === 'loading') {
    window.addEventListener('DOMContentLoaded', simplerInit);
  } else {
    simplerInit();
  }

  window.addEventListener('mini_cart_loaded', simplerInit);

  jQuery(document).ready(($) => {
    $(document.body).on('wc_fragments_loaded updated_cart_totals added_to_cart', (e) => {
      simplerInit();
    });
  });
})();

const SIMPLERWC_POPUP_WIDTH = 520;
const SIMPLERWC_POPUP_HEIGHT = 720;

class SimplerCheckout {
  constructor() {
    this.inputTarget = 'simpler-cart-payload';
    this.isValidCart = true;
  }

  toJSON() {
    const decoded = atob(document.getElementById(this.inputTarget).value);
    return JSON.parse(decoded);
  }

  checkout(e) {
    e.preventDefault();
    const order = this.buildOrder();
    const orderValid = this.validateOrder(order);
    if (!orderValid) {
      window.alert("Invalid product selection. Ensure product is in stock and all required options are selected");
      return false;
    }
    const url = this.buildUrl(e.currentTarget.href, order);
    const popup = this.openPopup(url);

    const overlay = buildOverlay(popup);
    document.body.classList.add("overlay-active");
    document.body.appendChild(overlay);

    const timer = setInterval(() => {
      if (popup.closed) {
        removeOverlay(overlay);
        clearInterval(timer);
      }
    }, 500);

    window.addEventListener('message', event => {
      if (event.data === 'simpler.orderCreated') {
        window.location = window.location.origin + '?simpler_order_created=1';
      }
    });

    return false;
  }

  openPopup(url) {
    const { left, top } = centerPopup();
    const newWindow = window.open(url, 'popup',
      [
        'scrollbars=yes',
        `width=${SIMPLERWC_POPUP_WIDTH}`,
        `height=${SIMPLERWC_POPUP_HEIGHT}`,
        `top=${top}`,
        `left=${left}`
      ].join(', ')
    );

    if (newWindow.focus) {
      newWindow.focus();
    }
    return newWindow;
  }

  buildUrl(href, order) {
    const items = order.items.map(item => {
      return {
        id: item.id,
        qty: item.quantity
      }
    })

    return `${href}&prd=${encodeURIComponent(JSON.stringify(items))}&cur=${order.currency}`;
  }

  buildOrder() {
    const { cart, currency } = this.toJSON();
    return {
      ...cart,
      items: cart.map(el => this.buildOrderItem(el)),
      currency
    };
  }

  validateOrder(order) {
    if (!this.isValidCart) {
      return false;
    }

    const atLeastOneItemInvalid = order.items.some(function (item) {
      const isInStock = item.in_stock || item.backorders_allowed;
      if (!isInStock) {
        return true;
      }

      return item.quantity <= 0 || (item.stock_quantity > 0 && item.quantity > item.stock_quantity && !item.backorders_allowed);
    })
    return !atLeastOneItemInvalid;
  }

  buildOrderItem(item) {
    return {
      id: item.product_id,
      quantity: parseInt(item.quantity, 10),
      in_stock: item.in_stock,
      backorders_allowed: item.backorders_allowed,
      stock_quantity: item.stock_quantity || 0,
    }
  }
}

class SimplerCheckoutSingle extends SimplerCheckout {
  constructor() {
    super();
    this.isValidCart = false;
    this.inputTarget = 'simpler-product-payload';
  }

  buildOrder() {
    const { cart, currency } = this.toJSON();
    const product = cart[0];
    const qty = document.querySelector('form.cart input[name="quantity"]').value;
    if (product.product_type === 'simple') {
      this.isValidCart = true;
      return {
        currency,
        items: [this.buildOrderItem({ ...product, quantity: qty })]
      };
    }

    const variation = this.getSelectedVariation(product);
    this.isValidCart = Boolean(parseInt(variation.product_id));
    return {
      currency,
      items: [this.buildOrderItem({
        ...product,
        ...variation,
        quantity: qty
      })]
    }
  };

  getSelectedVariation(product) {
    const variationId = document.querySelector(
      'form.cart input[name="variation_id"]'
    ).value;

    if (Boolean(parseInt(variationId))) {
      return product.variations.find(
        el => el.product_id.toString() === variationId.toString()
      ) || {};
    }

    return {};
  }
}

const centerPopup = () => {
  const dualScreenLeft = window.screenLeft != undefined ? window.screenLeft : screen.left;
  const dualScreenTop = window.screenTop != undefined ? window.screenTop : screen.top;

  const windowWidth = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth ? document.documentElement.clientWidth : screen.width;
  const windowHeight = window.innerHeight ? window.innerHeight : document.documentElement.clientHeight ? document.documentElement.clientHeight : screen.height;

  const left = ((windowWidth / 2) - (SIMPLERWC_POPUP_WIDTH / 2)) + dualScreenLeft;
  const top = ((windowHeight / 2) - (SIMPLERWC_POPUP_HEIGHT / 2)) + dualScreenTop;
  return { left, top };
}

const buildOverlay = popup => {
  const root = document.createElement("div");
  root.classList.add("simpler-overlay");

  const container = document.createElement("div");
  container.classList.add("simpler-overlay-container");

  const closeBtn = document.createElement("span");
  closeBtn.appendChild(document.createTextNode("\u2715"));
  closeBtn.classList.add("simpler-overlay-close");
  closeBtn.addEventListener('click', () => {
    popup.close();
    document.remove(root);
  });

  const lead = document.createElement("object");
  lead.data = "https://cdn.simpler.so/logos/simpler-logo-transparent-white.svg";
  lead.width = "200";
  lead.height = "30";
  lead.classList.add("simpler-overlay-lead");

  const notice = document.createElement("p");
  notice.appendChild(document.createTextNode("Don't see your checkout window?"));
  notice.classList.add("simpler-overlay-notice");

  const cta = document.createElement("a");
  cta.appendChild(document.createTextNode("Click here"));
  cta.href = "#";
  cta.role = "button";
  cta.classList.add("simpler-overlay-cta");
  cta.addEventListener('click', () => {
    popup.focus();
    popup.postMessage("simpler.centerSelf", "*");
  });

  root.appendChild(container);
  container.appendChild(closeBtn);
  container.appendChild(lead);
  container.appendChild(notice);
  container.appendChild(cta);

  return root;
}

const removeOverlay = overlay => {
  overlay.remove();
  document.body.classList.remove("overlay-active");
}
